# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, String

logger = logging.getLogger("driverng.amplidex")


class AmplidexResult(object):
    FIELDS = [
        String("sample"),
        String("gender"),
        String("well"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        for index, field in enumerate(self.FIELDS):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            setattr(self, field.name, line[index])


class AmplidexMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: AmplidexResult}

    def add(self, type_message=MessageV2.RESULT):
        obj = self._MESSAGE_TYPE_MAP[type_message]()
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.date_result = None
        self.operator = None
        self.batch = None
        for line in reader:
            if not len(line):
                continue
            if line[0].startswith("#Batch:"):
                self.batch = line[0].split(":")[1].strip()
            elif line[0].startswith("#Operator:"):
                self.operator = line[0].split(":")[1].strip()
            elif line[0].startswith("#Analyzed Date:"):
                try:
                    self.date_result = datetime.datetime.strptime(
                        line[0].split(":", 1)[1].strip(), "%m/%d/%Y %H:%M:%S %p"
                    )
                except ValueError as e:
                    logger.info(
                        "La date dans Analyzed Date n'est pas au bon format %s", e
                    )
            if len(line) > 4:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "amplidex", "label": "AMPLIDEX CSV"}

    def parse(self, data):
        obj = AmplidexMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.sample == "Sample":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                if m.sample in ("SensitivityControl", "ProcessControl"):
                    sample = worklist.get_or_create(
                        sample_id=m.sample, control_lot=m.sample, type="control"
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=m.sample, rack=m.well, type="patient"
                    )
                for index, result in enumerate(m.results):
                    sample.results.add(
                        code="{}{}".format(message.batch, RESULTS_CODES[index]),
                        value=result,
                        date=message.date_result,
                        operator=message.operator,
                    )
        return worklist
